#!/usr/bin/ruby

require 'fileutils'
require 'json'
require 'open-uri'
require 'optparse'
require 'socket'
require 'syslog/logger'

$hostname = Socket.gethostname

$log = Syslog::Logger.new 'storage_stats.rb'

$output = []

storage_groups_files = ''

$affiliation_file = nil
$mu_mode = false
$gpfs_mode = false
$volume = nil

wpmap = {
  '1411' => 'LF-1411',
  '1421' => 'FF-1421',
  '1422' => 'LAW-1422',
  '1423' => 'FSS-1423',
  '1427' => 'MPÚ-1427',
  '1431' => 'SCI-1431',
  '1433' => 'FI-1433',
  '1441' => 'PedF-1441',
  '1451' => 'FSpS-1451',
  '1456' => 'ESF-1456',
  '1458' => 'ArchMU-1458',
  '1471' => 'Ceitec-1471',
  '1479' => 'Ceitec-1479',
  '1481' => 'SKM-1481',
  '1482' => 'SUKB-1482',
  '1483' => 'UCT-1483',
  '1484' => 'SSN-1484',
  '1485' => 'IBA-1485',
  '1486' => 'MenMu-1486',
  '1487' => 'CTT-1487',
  '1488' => 'KC-1488',
  '1489' => 'CRPK-1489',
  '1490' => 'CUSt-1490',
  '1492' => 'ÚVT-1492',
  '1494' => 'Nakladatelství-1494',
  '1496' => 'CJV-1496',
  '1497' => 'CZS-1497',
  '1499' => 'RMU-1499',
}


OptionParser.new do |opts|
  opts.banner = "Usage: storage_stats.rb [options]"

  opts.on("-A", "--auto-groups", "Use autogroups dir") do |v|
    storage_groups_files = `cat /etc/auto-groups.d/*`
  end

  opts.on("-aNAME", "--affiliation-file=NAME", "Affiliation mapping location (fullpath with filename)") do |v|
    $affiliation_file = v
  end

  opts.on("-m", "--mu", "Use MUNI mode for UCOs handlig") do |v|
    $mu_mode = true
  end

  opts.on("-HHOSTNAME", "--hostname=HOSTNAME", "Force use hostname instead of detecting") do |v|
    $hostname = v
  end

  opts.on("-g", "--gpfs", "Use gpfs parsing") do |v|
    $gpfs_mode = v
  end

  opts.on("-VVOLUME", "--volume=VOLUME", "Collect stats for VOLUME") do |v|
    $volume = v
  end
end.parse!

exit 0 if $affiliation_file == nil or $volume == nil

storage_groups = {}

storage_groups_files.each_line do |line|
  grps = line.split(':')[0].split('|')
  grps.each do |grp|
    storage_groups[grp] = 1
  end
end

orgs = open(URI('https://metavo.metacentrum.cz/pbsmon2/api/org_names'))

exit 0 if orgs == nil;

begin
  orgs_a = JSON.parse(File.read(orgs.path).force_encoding('UTF-8'))
rescue StandardError => msg
  $log.error("JSON parse orgs error: #{msg}")
  exit 0
end

orgs_h = {}

orgs_a.each do |org|
  orgs_h[org['user_org']] = org['name']
end

begin
  mappings = JSON.parse(File.read($affiliation_file).force_encoding('UTF-8'))
rescue StandardError => msg
  $log.error("affiliation file error: #{msg}")
  exit 0
end

af = {}

mappings.each do |item| 
   login = item['urn:perun:user_facility:attribute-def:virt:login']
   workplaceid = item['urn:perun:user:attribute-def:def:workplaceId']
   facultyid = item['urn:perun:user:attribute-def:def:enrolledFacultyID']
   deforg = item['urn:perun:user:attribute-def:def:organization']
   if item['urn:perun:user:attribute-def:virt:userOrganizations'] != nil
      metaorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['meta']
      einfraorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['einfra']
      elixirorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['elixir-cz']
   end
   if metaorg != nil
      wplace = metaorg
   elsif einfraorg != nil
      wplace = einfraorg
   elsif elixirorg != nil
      wplace = elixirorg
   else
      wplace = deforg
   end

   if orgs_h[wplace] == nil
     $log.warn("unknown org: #{wplace}") if wplace != nil
     wplace = 'Others' 
   else
     wplace = orgs_h[wplace]
   end
   if login =~ /^9....../ 
      wplace = 'Guests'
   end
   if workplaceid != nil
      wplace = workplaceid
   elsif facultyid != nil
      wplace = facultyid
   end
   af[login] = wplace if af[login] == nil
end

expired = {}

File.read('/etc/shadow').each_line do |line|
  items = line.split(':')
  if items[7] == "1"
     expired[items[0]] = 1
  end
end

af['root'] = 'root'

if $mu_mode
  af['nakladatelstvi'] = '9400'
  af['s-uebbiol'] = '314010'
  af['scibotautologon'] = '310000'
  af['s-brigadnik'] = '8250'
  af['s-matl'] = '712000'
end

if $gpfs_mode 
  _gpfs_vol = $volume.gsub(/.*\//, '') 
  quotas = `/usr/lpp/mmfs/bin/mmrepquota #{_gpfs_vol} | sed -e 's/ [ ]*/ /g'`
else
  quotas = `repquota #{$volume} | sed -e 's/ [ ]*/ /g'`
end

total = 0

date=Time.now.to_i

org_used = {}

groupnum=0

if $gpfs_mode
  quotas.each_line do |line|
    items = line.split(' ')
    if items[1] == 'root'
      items[1] = 'shrd'
    end
    if items[1] == 'Archive'
      items[1] = 'arch'
    end
    if items[2] == 'USR' and items[3].to_i > 0
       total += items[3].to_i
       af[items[0]] = 'nil' if af[items[0]] == nil     
       org_used[af[items[0]]] = 1
       org = af[items[0]].gsub(/ /, '_')
       org = 'Others' if org == nil or org == 'nil'
       if $mu_mode
         if org == 'Others' and File.directory?("#{$volume}/export/home/#{items[0]}")
           org = 'MU'
         end
         groups = `groups #{items[0]} 2> /dev/null`.chomp
         if af[items[0]] == nil
           if items[0] =~ /guest/
             af[items[0]] = 'Guests'
           elsif groups =~ /wplace-[1-9]/
             af[items[0]] = groups.gsub(/.*wplace-([1-9][0-9]*).*/, '\1')
           elsif groups =~ /ceitec-[1-9]/
             af[items[0]] = groups.gsub(/.*ceitec-([1-9][0-9]*).*/, '\1')
           else
             af[items[0]] = 'nil'
           end
         end
       end
    
       wid = af[items[0]].to_s
       if wid =~ /^[0-9]*$/ and $mu_mode
         wp = '14'+wid.gsub(/(..).*/,'\1')
         widtext = wpmap[wp]
         widtext = wp if widtext == nil
       else
         widtext = wid
       end

       if widtext == 'nil'
         widtext = 'No_affiliation'
       end

       if expired[items[0]] != nil
         $output << {'name' => 'diskspace', 'value' => items[3].to_i*1024,
                   'host' => $hostname, 'time' => date,
                   'wid' => org, 'widtext' => widtext, 'user' => items[0], 'subvol' => items[1], 'expired' => 1 }
       else
         $output << {'name' => 'diskspace', 'value' => items[3].to_i*1024,
                   'host' => $hostname, 'time' => date,
                   'wid' => org, 'widtext' => widtext, 'user' => items[0], 'subvol' => items[1], 'expired' => 0 }
       end
    end
  end
else
  $_start = false
  quotas.each_line do |line|
    _start = true if line =~ /-----------/
    next if _start == false
    items = line.split(' ')
    if items[2].to_i > 0
      af[items[0]] = 'nil' if af[items[0]] == nil
      org_used[af[items[0]]] = 1
      org = af[items[0]].gsub(/ /, '_')
      org = 'Others' if org == nil or org == 'nil'

      wid = af[items[0]].to_s
      if wid =~ /^[0-9]*$/ and $mu_mode
        wp = '14'+wid.gsub(/(..).*/,'\1')
        widtext = wpmap[wp]
        widtext = wp if widtext == nil
      else
        widtext = wid
      end

      if widtext == 'nil'
        widtext = 'No_affiliation'
      end

      if expired[items[0]] != nil
        $output << {'name' => 'diskspace', 'value' => items[3].to_i*1024,
                  'host' => $hostname, 'time' => date,
                  'wid' => org, 'widtext' => widtext, 'user' => items[0], 'subvol' => '', 'expired' => 1 }
      else
        $output << {'name' => 'diskspace', 'value' => items[3].to_i*1024,
                  'host' => $hostname, 'time' => date,
                  'wid' => org, 'widtext' => widtext, 'user' => items[0], 'subvol' => '', 'expired' => 0 }
      end
    end
  end
end

puts(JSON.pretty_generate($output))
